CREATE ROLE "USERCHARACTERBUILDER" WITH ENCRYPTED PASSWORD 'PASSWORDCHARACTERBUILDER' LOGIN;

CREATE SCHEMA characterbuilder
  AUTHORIZATION "USERCHARACTERBUILDER";
  
CREATE TABLE characterbuilder.user(
	id serial,
	username text NOT NULL,
	password text,
	email text,
	CONSTRAINT "user_pk" PRIMARY KEY (id),
	CONSTRAINT "user_name_un" UNIQUE (username)
);

CREATE TABLE characterbuilder.role(
	id serial,
	name text NOT NULL,
	CONSTRAINT "role_pk" PRIMARY KEY (id),
	CONSTRAINT "role_name_un" UNIQUE (name)
);

CREATE TABLE characterbuilder.user_role(
	id_user integer not null,
	id_role integer not null,
	CONSTRAINT "user_role_pk" PRIMARY KEY (id_user, id_role)
);

ALTER TABLE characterbuilder.user_role ADD CONSTRAINT "user_role_fk1" FOREIGN KEY (id_user)
REFERENCES characterbuilder.user (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

ALTER TABLE characterbuilder.user_role ADD CONSTRAINT "user_role_fk2" FOREIGN KEY (id_role)
REFERENCES characterbuilder.role (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

/*Quirks(virtuts i defectes) i les seves categoríes*/

CREATE TABLE characterbuilder.quirkcategory(
	id serial,
	name text NOT NULL,
	CONSTRAINT "quirkCategory_pk" PRIMARY KEY (id),
	CONSTRAINT "quirkCategory_name_un" UNIQUE (name)
);

CREATE TABLE characterbuilder.quirk(
	id serial,
	name text NOT NULL,
	description text,
	quirkdivision text NOT NULL,-- Virtue or flaw
	quirktype text NOT NULL, -- Major or minor
	CONSTRAINT "quirk_pk" PRIMARY KEY (id),
	CONSTRAINT "quirk_name_division_type_un" UNIQUE (name, quirkdivision, quirktype)
);

CREATE TABLE characterbuilder.quirkcategory_quirk(
	id_quirkcategory integer,
	id_quirk integer,
	CONSTRAINT "quirkCategory_quirk_pk" PRIMARY KEY (id_quirkcategory, id_quirk)
);

ALTER TABLE characterbuilder.quirkcategory_quirk ADD CONSTRAINT "quirkCategory_quirk_fk1" FOREIGN KEY (id_quirk)
REFERENCES characterbuilder.quirk (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

ALTER TABLE characterbuilder.quirkcategory_quirk ADD CONSTRAINT "quirkCategory_quirk_fk2" FOREIGN KEY (id_quirkcategory)
REFERENCES characterbuilder.quirkcategory (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

/*Relacions de dependencia entre quirks i altres elements*/

/*Relacio entre dos quirks*/

CREATE TABLE characterbuilder.quirk_quirk(
	id serial,
	id_dependent_quirk integer,
	id_required_quirk integer,
	negated boolean default false,
	CONSTRAINT "quirk_quirk_pk" PRIMARY KEY (id),
	CONSTRAINT "quirk_quirk_name_un" UNIQUE (id_dependent_quirk,id_required_quirk)
);

ALTER TABLE characterbuilder.quirk_quirk ADD CONSTRAINT "quirk_quirk_fk1" FOREIGN KEY (id_dependent_quirk)
REFERENCES characterbuilder.quirk (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

ALTER TABLE characterbuilder.quirk_quirk ADD CONSTRAINT "quirk_quirk_fk2" FOREIGN KEY (id_required_quirk)
REFERENCES characterbuilder.quirk (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

/*Categories d'habilitats*/

CREATE TABLE characterbuilder.abilitycategory(
	id serial,
	name text not null,
	CONSTRAINT "abilitycategory_pk" PRIMARY KEY (id),
	CONSTRAINT "abilitycategory_un" UNIQUE (name)
);

CREATE TABLE characterbuilder.abilitycategory_quirk(
	id serial,
	id_abilitycategory integer,
	id_quirk integer,
	negated boolean default false,
	CONSTRAINT "abilitycategory_quirk_pk" PRIMARY KEY (id),
	CONSTRAINT "abilitycategory_quirk_un" UNIQUE (id_abilitycategory,id_quirk)
);

ALTER TABLE characterbuilder.abilitycategory_quirk ADD CONSTRAINT "abilitycategory_quirk_fk1" FOREIGN KEY (id_abilitycategory)
REFERENCES characterbuilder.abilitycategory (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

ALTER TABLE characterbuilder.abilitycategory_quirk ADD CONSTRAINT "abilitycategory_quirk_fk2" FOREIGN KEY (id_quirk)
REFERENCES characterbuilder.quirk (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

/*Habilitats*/

CREATE TABLE characterbuilder.ability(
	id serial,
	name text not null,
	description text,
	infancySelectable boolean default false,
	CONSTRAINT "ability_pk" PRIMARY KEY (id),
	CONSTRAINT "ability_un" UNIQUE (name)
);

CREATE TABLE characterbuilder.ability_abilitycategory (
	id_ability integer,
	id_abilitycategory integer,
	CONSTRAINT "ability_abilitycategory_pk" PRIMARY KEY (id_ability, id_abilitycategory)
);

ALTER TABLE characterbuilder.ability_abilitycategory ADD CONSTRAINT "ability_abilitycategory_fk1" FOREIGN KEY (id_ability)
REFERENCES characterbuilder.ability (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

ALTER TABLE characterbuilder.ability_abilitycategory ADD CONSTRAINT "ability_abilitycategory_fk2" FOREIGN KEY (id_abilitycategory)
REFERENCES characterbuilder.abilitycategory (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

CREATE TABLE characterbuilder.ability_quirk(
	id serial,
	id_ability integer not null,
	id_quirk integer not null,
	negated boolean default false,
	CONSTRAINT "ability_quirk_pk" PRIMARY KEY (id),
	CONSTRAINT "ability_quirk_un" UNIQUE (id_ability,id_quirk)
);

ALTER TABLE characterbuilder.ability_quirk ADD CONSTRAINT "ability_quirk_fk1" FOREIGN KEY (id_ability)
REFERENCES characterbuilder.ability (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

ALTER TABLE characterbuilder.ability_quirk ADD CONSTRAINT "ability_quirk_fk2" FOREIGN KEY (id_quirk)
REFERENCES characterbuilder.quirk (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

/*Personatges*/

CREATE TABLE characterbuilder.character(
	id serial,
	name text not null,
	description text,
	gender text not null,
	age integer not null,
	str integer default 0,
	sta integer default 0,
	per integer default 0,
	ine integer default 0,
	dex integer default 0,
	qui integer default 0,
	cha integer default 0,
	com integer default 0,
	pendingrevission boolean default false,
	CONSTRAINT "character_pk" PRIMARY KEY (id)
);

/*Relacions entre personatges i usuaris*/

CREATE TABLE characterbuilder.user_character(
	id serial,
	id_user integer not null,
	id_character integer not null,
	owner boolean not null,
	canread boolean not null,
	canwrite boolean not null,
	CONSTRAINT "user_character_pk" PRIMARY KEY (id),
	CONSTRAINT "user_character_un1" UNIQUE (id_user,id_character)
);

ALTER TABLE characterbuilder.user_character ADD CONSTRAINT "user_character_fk1" FOREIGN KEY (id_user)
REFERENCES characterbuilder.user (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

ALTER TABLE characterbuilder.user_character ADD CONSTRAINT "user_character_fk2" FOREIGN KEY (id_character)
REFERENCES characterbuilder.character (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

/*Abilitats associades al personatge*/

CREATE TABLE characterbuilder.ability_character(
	id serial,
	id_ability integer not null,
	id_character integer not null,
	infancyinvestedxps integer default 0,
	adultinvestedxps integer default 0,
	CONSTRAINT "ability_character_pk" PRIMARY KEY (id),
	CONSTRAINT "ability_characterk_un" UNIQUE (id_ability,id_character)
);

ALTER TABLE characterbuilder.ability_character ADD CONSTRAINT "ability_character_fk1" FOREIGN KEY (id_ability)
REFERENCES characterbuilder.ability (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

ALTER TABLE characterbuilder.ability_character ADD CONSTRAINT "ability_character_fk2" FOREIGN KEY (id_character)
REFERENCES characterbuilder.character (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

/*Relacions entre personatges i quirks*/

CREATE TABLE characterbuilder.character_quirk(
	id_character integer not null,
	id_quirk integer not null,
	CONSTRAINT "character_quirk_pk" PRIMARY KEY (id_character, id_quirk)
);

ALTER TABLE characterbuilder.character_quirk ADD CONSTRAINT "character_quirk_fk1" FOREIGN KEY (id_character)
REFERENCES characterbuilder.character (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

ALTER TABLE characterbuilder.character_quirk ADD CONSTRAINT "character_quirk_fk2" FOREIGN KEY (id_quirk)
REFERENCES characterbuilder.quirk (id) MATCH FULL
ON DELETE NO ACTION ON UPDATE NO ACTION NOT DEFERRABLE;

/*Log d'accions*/

CREATE TABLE characterbuilder.log(
	id serial,
	logdate date NOT NULL,
	description text NOT NULL,
	CONSTRAINT "log_pk" PRIMARY KEY (id)
);

GRANT ALL PRIVILEGES ON SCHEMA characterbuilder TO "USERCHARACTERBUILDER";

GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA characterbuilder TO "USERCHARACTERBUILDER";

GRANT ALL PRIVILEGES ON ALL SEQUENCES IN SCHEMA characterbuilder TO "USERCHARACTERBUILDER";