package edu.uoc.characterbuilder.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;
import javax.persistence.PersistenceException;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import edu.uoc.characterbuilder.ejb.AbilityCategoryEJBLocal;
import edu.uoc.characterbuilder.ejb.QuirkCategoryEJBLocal;
import edu.uoc.characterbuilder.model.AbilityCategoryJPA;
import edu.uoc.characterbuilder.model.QuirkCategoryJPA;

@Controller
public class AbilityCategoryController {

	@Inject
	private AbilityCategoryEJBLocal abilityCategoryEJB;
	
	@Inject
	private QuirkCategoryEJBLocal quirkCategoryEJB;
	
	@RequestMapping(value="/admin/abilityCategories", method=RequestMethod.GET)
	public String abilityCategories(Map<String, Object> model) {
		
		List<AbilityCategoryJPA> abilityCategoryList = abilityCategoryEJB.getAllAbilityCategories();
		
		List<QuirkCategoryJPA> quirkCategoryList = quirkCategoryEJB.getAllQuirkCategories();
		
		model.put("AbilitycategoryList", abilityCategoryList);
		model.put("categoryList", quirkCategoryList);
		
		return "admin/views/abilityCategoryView";
	}

	@RequestMapping(value="/admin/deleteAbilityCategory", method=RequestMethod.POST)
	@ResponseBody public Map<String, Object> deleteAbilityCategory(@RequestParam int id) {
		
		Map<String, Object> result = new HashMap<String, Object>();;
		
		result.put("error","none");
		
		try {
			result = abilityCategoryEJB.deleteAbilityCategoryById(id);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "database_error");
		}
		
		return result;
	}
	
	@RequestMapping(value="/admin/saveNewAbilityCategory", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> saveNewAbilityCategory(@RequestBody Map<String, Object> newAbilityCategoryData) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error","none");
		
		try {
			abilityCategoryEJB.newAbilityCategory(newAbilityCategoryData);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "Ability category " + newAbilityCategoryData.get("name") + " already existent");
		}
		
		return result;
	}

	@RequestMapping(value="/admin/getAbilityCategory", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> getAbilityCategoryById(@RequestParam int id) {
		
		AbilityCategoryJPA abilityCategory = abilityCategoryEJB.getAbilityCategoryById(id);

		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("abilityCategory", abilityCategory);
		result.put("requiredQuirks", abilityCategory.getRequiredQuirks());
		result.put("error", "none");
		
		return result;

	}
	
	@RequestMapping(value="/admin/updateAbilityCategory", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> updateAbilityCategory(@RequestBody Map<String, Object> newAbilityCategoryData) {
		
		Map<String, Object> result = new HashMap<String, Object>();;
		
		result.put("error","none");
		
		try {
			result = abilityCategoryEJB.updateAbilityCategory(newAbilityCategoryData);
		} catch (PersistenceException e) {
			result.put("error", "Ability category " + newAbilityCategoryData.get("name") + " already existent");
		}
		
		return result;
	}
	
}
