package edu.uoc.characterbuilder.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;
import javax.persistence.PersistenceException;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import edu.uoc.characterbuilder.ejb.AbilityCategoryEJBLocal;
import edu.uoc.characterbuilder.ejb.AbilityEJBLocal;
import edu.uoc.characterbuilder.ejb.QuirkCategoryEJBLocal;
import edu.uoc.characterbuilder.model.AbilityCategoryJPA;
import edu.uoc.characterbuilder.model.AbilityJPA;
import edu.uoc.characterbuilder.model.QuirkCategoryJPA;

@Controller
public class AbilityController {

	@Inject
	private AbilityCategoryEJBLocal abilityCategoryEJB;
	
	@Inject
	private AbilityEJBLocal abilityEJB;
	
	@Inject
	private QuirkCategoryEJBLocal quirkCategoryEJB;
	
	@RequestMapping(value="/admin/abilities", method=RequestMethod.GET)
	public String abilities(Map<String, Object> model) {
		
		List<AbilityCategoryJPA> abilityCategoryList = abilityCategoryEJB.getAllAbilityCategories();
		
		List<QuirkCategoryJPA> quirkCategoryList = quirkCategoryEJB.getAllQuirkCategories();

		model.put("abilityCategoryList", abilityCategoryList);
		model.put("quirkCategoryList", quirkCategoryList);
		
		return "admin/views/abilityView";
	}

	@RequestMapping(value="/admin/deleteAbility", method=RequestMethod.POST)
	@ResponseBody public Map<String, Object> deleteAbility(@RequestParam int id) {
		
		Map<String, Object> result = null;
		
		try {
			result = abilityEJB.deleteAbilityById(id);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result = new HashMap<String, Object>();
			result.put("error", "database_error");
		}
		
		return result;
	}
	
	@RequestMapping(value="/admin/saveNewAbility", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> saveNewAbility(@RequestBody Map<String, Object> newAbilityData) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error","none");
		
		try {
			result= abilityEJB.newAbility(newAbilityData);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "Ability " + newAbilityData.get("name") + " already existent");
		}
		
		return result;
	}

	@RequestMapping(value="/admin/getAbility", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> getAbilityById(@RequestParam int id) {
		
		AbilityJPA ability = abilityEJB.getAbilityById(id);

		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("ability", ability);
		result.put("requiredQuirks", ability.getRequiredQuirks());
		result.put("error", "none");
		
		return result;

	}
	
	@RequestMapping(value="/admin/updateAbility", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> updateAbility(@RequestBody Map<String, Object> newAbilityData) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error","none");
		
		try {
			result = abilityEJB.updateAbility(newAbilityData);
		} catch (javax.persistence.PersistenceException e) {
			result.put("error", "Ability " + newAbilityData.get("name") + " already existent");
		}
		
		return result;
	}
	
}
