package edu.uoc.characterbuilder.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;

import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import edu.uoc.characterbuilder.ejb.AbilityCategoryEJBLocal;
import edu.uoc.characterbuilder.ejb.CharacterEJBLocal;
import edu.uoc.characterbuilder.ejb.QuirkCategoryEJBLocal;
import edu.uoc.characterbuilder.ejb.UserManagementEJBLocal;
import edu.uoc.characterbuilder.model.AbilityCategoryJPA;
import edu.uoc.characterbuilder.model.CharacterJPA;
import edu.uoc.characterbuilder.model.QuirkCategoryJPA;
import edu.uoc.characterbuilder.model.UserJPA;

@Controller
public class CharacterController {
	
	@Inject
	private CharacterEJBLocal characterEJB;
	
	@Inject
	private UserManagementEJBLocal userEJB;
	
	@Inject
	private QuirkCategoryEJBLocal quirkCategoryEJB;
	
	@Inject
	private AbilityCategoryEJBLocal abilityCategoryEJB;
	
	@RequestMapping(value="/characters", method=RequestMethod.GET)
	public String characters(Map<String, Object> model) {
		
		Authentication loggedUser = SecurityContextHolder.getContext().getAuthentication();
		
		List<CharacterJPA> characterList = characterEJB.getAccessibleCharactersByUserName(loggedUser.getName());
		
		List<QuirkCategoryJPA> quirkCategoryList = quirkCategoryEJB.getAllQuirkCategories();
		
		List<AbilityCategoryJPA> abilityCategoryList = abilityCategoryEJB.getAllAbilityCategories();
		
		model.put("characterList", characterList);
		model.put("quirkCategoryList", quirkCategoryList);
		model.put("abilityCategoryList", abilityCategoryList);
		model.put("loggedUser", loggedUser.getName());
		
		return "views/characterView";
	}

	@RequestMapping(value="/updateCharacter", method=RequestMethod.POST)
	@ResponseBody public Map<String, Object> updateCharacter(@RequestBody HashMap<String, Object> newCharacterData) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error", "none");
		
		try {
			characterEJB.saveUpdatedCharacter(newCharacterData);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "existing_element");
		}
		
		return result;
	}
	
	@RequestMapping(value="/getNonOwners", method=RequestMethod.POST)
	@ResponseBody public Map<String, Object> getNonOwners(@RequestParam int id) {
		
		List<UserJPA> userList = userEJB.getNonOwners(id);
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("userList", userList);
		
		return result;
	}
	
	@RequestMapping(value="/updateRights", method=RequestMethod.POST)
	@ResponseBody public Map<String, Object> updateRights(@RequestBody HashMap<String, Object> accessRights) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error", "none");
		
		try {
			characterEJB.updateRights(accessRights);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "database_error");
		}
		
		return result;
	}
	
	@RequestMapping(value="/getCharacter", method=RequestMethod.POST)
	@ResponseBody public Map<String, Object> getCharacter(@RequestParam int id) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error", "none");
		
		try {
			result.put("character", characterEJB.getCharacterById(id));
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "database_error");
		}
		
		return result;
	}
	
	@RequestMapping(value="/deleteCharacter", method=RequestMethod.POST)
	@ResponseBody public Map<String, Object> deleteCharacter(@RequestParam int id) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error", "none");
		
		try {
			characterEJB.deleteCharacterById(id);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "database_error");
		}
		
		return result;
	}
	
	@RequestMapping(value="/makeUserOwner", method=RequestMethod.POST)
	@ResponseBody public Map<String, Object> makeUserOwner(@RequestParam Map<String, Object> data) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error", "none");
		
		try {
			characterEJB.makeUserOwner(data);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "database_error");
		}
		
		return result;
	}
	
	@RequestMapping(value="/saveNewCharacter", method=RequestMethod.POST)
	@ResponseBody public Map<String, Object> saveNewCharacter(@RequestBody HashMap<String, Object> newCharacterData) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error", "none");
		
		try {
			characterEJB.saveNewCharacter(newCharacterData);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "existing_element");
		}
		
		return result;
	}
	
	@RequestMapping(value="/validateCharacter", method=RequestMethod.POST)
	@ResponseBody public Map<String, Object> validateCharacter(@RequestBody HashMap<String, Object> newCharacterData) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error", "none");
		
		try {
			result.put("validationResult", characterEJB.validateCharacter(newCharacterData));
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "existing_element");
		}
		
		return result;
	}
	
	@RequestMapping(value="/getUpdatedElementsLists", method=RequestMethod.POST)
	@ResponseBody public Map<String, Object> getUpdatedElementsLists(@RequestBody List<Integer> chosenQuirks) {

		
		return characterEJB.getUpdatedElementsLists(chosenQuirks);
		
	}
	
}
