package edu.uoc.characterbuilder.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import edu.uoc.characterbuilder.ejb.QuirkCategoryEJBLocal;
import edu.uoc.characterbuilder.model.QuirkCategoryJPA;
import edu.uoc.characterbuilder.model.QuirkJPA;

@Controller
public class QuirkCategoryController {

	@Inject
	private QuirkCategoryEJBLocal categoryEJB;
	
	@RequestMapping(value="/admin/getQuirkCategory", method=RequestMethod.POST)
	@ResponseBody
	public QuirkCategoryJPA getQuirkCategoryById(@RequestParam int id) {
		
		QuirkCategoryJPA quirkCategory = categoryEJB.getQuirkCategoryById(id);

		return quirkCategory;

	}
	
	@RequestMapping(value="/admin/quirkCategories", method=RequestMethod.GET)
	public String quirkCategories(Map<String, Object> model) {
		
		List<QuirkCategoryJPA> categoryList = categoryEJB.getAllQuirkCategories();
		
		model.put("categoryList", categoryList);
		
		return "admin/views/quirkCategoryView";
	}
	
	//El tag @ResponseBody informa al spring que se retorna un objecte serialitzat, no un link a una vista 
	@RequestMapping(value="/admin/deleteQuirkCategory", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> deleteQuirkCategory(@RequestParam int id) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		List<QuirkJPA> listQuirks = categoryEJB.deleteQuirkCategoryById(id);
		
		if (listQuirks.isEmpty()) {
			result.put("result", "correct");
		} else {
			result.put("result", "notEmpty");
			result.put("quirks", listQuirks);
		}
		
		return result;
	}
	
	@RequestMapping(value="/admin/saveNewQuirkCategory", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> saveNewQuirkCategory(@RequestParam Map<String, String> newQuirkCategoryData) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error","none");
		
		try {
			categoryEJB.newQuirkCategory(newQuirkCategoryData);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "Quirk category " + newQuirkCategoryData.get("name") + " already existent");
		} 
		result.put("elements", categoryEJB.getAllQuirkCategories());
		
		return result;
	}
	
	@RequestMapping(value="/admin/updateQuirkCategory", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> updateQuirkCategory(@RequestParam Map<String, String> newQuirkCategoryData) {

		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error","none");
		
		try {
			categoryEJB.updateQuirkCategory(newQuirkCategoryData);
		} catch (javax.persistence.PersistenceException e) {
			result.put("error","Quirk category " + newQuirkCategoryData.get("name") + " already existent");
		} 
		result.put("elements", categoryEJB.getAllQuirkCategories());
		
		return result;
	}

}
