package edu.uoc.characterbuilder.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import edu.uoc.characterbuilder.ejb.QuirkCategoryEJBLocal;
import edu.uoc.characterbuilder.ejb.QuirkEJBLocal;
import edu.uoc.characterbuilder.model.QuirkCategoryJPA;
import edu.uoc.characterbuilder.model.QuirkJPA;

@Controller
public class QuirkController {

	@Inject
	private QuirkEJBLocal quirkEJB;
	@Inject
	private QuirkCategoryEJBLocal categoryEJB;
	
	
	@RequestMapping(value="/admin/quirks", method=RequestMethod.GET)
	public String quirkCategories(Map<String, Object> model) {
		
		List<QuirkCategoryJPA> categoryList = categoryEJB.getAllQuirkCategories();
		
		model.put("categoryList", categoryList);
		
		return "admin/views/quirksView";
	}
	
	@RequestMapping(value="/admin/deleteQuirk", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> deleteQuirk(@RequestParam int id) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("result", "correct");
		
		try {
			quirkEJB.deleteQuirkById(id);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			e.printStackTrace();
			result.put("result", "error");
		}
		
		return result;
	}
	
	@RequestMapping(value="/admin/getQuirk", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> getQuirkById(@RequestParam int id) {
		
		QuirkJPA quirk = quirkEJB.getQuirkById(id);

		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("quirk", quirk);
		result.put("requiredQuirks", quirk.getRequiredQuirks());
		result.put("dependentQuirk", quirk.getDependentQuirks());
		result.put("dependentAbilityCategory", quirk.getDependentAbilityCategory());
		result.put("dependentAbility", quirk.getDependentAbility());
		
		return result;

	}
	
	
	@RequestMapping(value="/admin/saveNewQuirk", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> saveNewQuirk(@RequestBody Map<String, Object> newQuirkData) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error","none");
		
		try {
			quirkEJB.newQuirk(newQuirkData);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "Quirk " + newQuirkData.get("name") + " already existent");
		}
		
		return result;
	}
	
	@RequestMapping(value="/admin/updateQuirk", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> updateQuirk(@RequestBody Map<String, Object> newQuirkData) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error","none");
		
		try {
			quirkEJB.updateQuirk(newQuirkData);
		} catch (javax.persistence.PersistenceException e) {
			result.put("error", "Quirk " + newQuirkData.get("name") + " already existent");
		}
		
		return result;
	}
}
