package edu.uoc.characterbuilder.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;

import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import edu.uoc.characterbuilder.ejb.UserManagementEJBLocal;
import edu.uoc.characterbuilder.model.UserJPA;

/*
 El tag Controller declara aquesta classe com a controlador de Spring
 El RequestMapping de classe diu que les requests sense prefixe acaben aqui
 El requestMapping de metode senyala que qualsevol peticio acabada amb crearUsuari.mvc
 I que no tingui cap prefixe p.e. /administracio/crearUsuari.mvc va aqui
 */
@Controller
public class UserController {

	@Inject
	private UserManagementEJBLocal userEJB;

	@RequestMapping(value="/admin/users", method=RequestMethod.GET)
	public String userManagement(Map<String, Object> model) {
	
		List<UserJPA> userList = userEJB.getAllPLayers();
		
		model.put("userList", userList);
		
		return "admin/views/userManagementView";
	}
	
	//El tag @ResponseBody informa al spring que se retorna un objecte serialitzat, no un link a una vista 
	@RequestMapping(value="admin/deleteUser", method=RequestMethod.POST)
	@ResponseBody
	public String deleteUser(@RequestParam int id) {
		
		userEJB.deleteUserById(id);
		
		return "correct";
	}
	
	@RequestMapping(value="/admin/saveNewUser", method=RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> saveNewUser(@RequestParam Map<String, String> newUserData) {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		result.put("error","none");
		
		try {
			userEJB.newUser(newUserData);
		} catch (org.springframework.transaction.UnexpectedRollbackException e) {
			result.put("error", "User " + newUserData.get("username") + " already existent");
		} 
		result.put("players", userEJB.getAllPLayers());
		
		return result;
	}

	@RequestMapping(value="/editUser", method=RequestMethod.GET)
	public String editUser(@RequestParam String username, Map<String, Object> model) {
		
		UserJPA user = userEJB.getUserByUsername(username);
		
		model.put("user", user);
		
		Authentication loggedUser = SecurityContextHolder.getContext().getAuthentication();
		
		if (loggedUser.getName().equals(username)){
			model.put("linkCancel", "/mainView");
		} else {
			model.put("linkCancel","/admin/users");	
		}

		return "views/editUserView";

	}
	
	@RequestMapping(value="/updateUser", method=RequestMethod.POST)
	@ResponseBody
	public String updateUserData(@RequestParam Map<String, String> newUserData) {

		try {
			userEJB.updateUser(newUserData);
		} catch (javax.persistence.PersistenceException e) {
			e.printStackTrace();
			return( "User " + newUserData.get("username") + " already existent");
		} 
		
		return "correct";
	}

}
