package edu.uoc.characterbuilder.ejb;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.UnexpectedRollbackException;
import org.springframework.transaction.annotation.Transactional;

import edu.uoc.characterbuilder.model.AbilityCategoryJPA;
import edu.uoc.characterbuilder.model.AbilityCategoryQuirkJPA;
import edu.uoc.characterbuilder.model.AbilityJPA;
import edu.uoc.characterbuilder.model.AbilityQuirkJPA;
import edu.uoc.characterbuilder.model.CharacterAbilityJPA;
import edu.uoc.characterbuilder.model.QuirkJPA;

@Named
@Stateless
public class AbilityCategoryEJB implements AbilityCategoryEJBLocal {

	private static final Logger logger = LoggerFactory
			.getLogger(AbilityCategoryEJB.class);

	@PersistenceContext(unitName = "characterBuilderPU")
	private EntityManager entman;
	
	@Inject
	private LogEJBLocal logEJB;
	
	@Inject
	private QuirkEJBLocal quirkEJB;
	
	@Override
	public List<AbilityCategoryJPA> getAllAbilityCategories() {

		List<AbilityCategoryJPA> abilityCategoriesList = entman.createQuery("from AbilityCategoryJPA a order by a.name",AbilityCategoryJPA.class).getResultList();

		return abilityCategoriesList;
	}

	@Override
	@Transactional
	public Map<String, Object> deleteAbilityCategoryById(int id) throws org.springframework.transaction.UnexpectedRollbackException {
		
		AbilityCategoryJPA abilityCategory = entman.createQuery("from AbilityCategoryJPA a where a.id = ?", AbilityCategoryJPA.class).setParameter(1, id).getSingleResult();
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		if(abilityCategory.getAbilities().isEmpty()){
			entman.createQuery("delete from AbilityCategoryQuirkJPA q where q.dependentAbilityCategoryJPA.id = ?").setParameter(1, id).executeUpdate();
			entman.createQuery("delete from AbilityCategoryJPA q where q.id = ?").setParameter(1, id).executeUpdate();
			
			logEJB.insertLog("Ability category deleted : " + abilityCategory.toString());
			logger.info("AbilityCategoryEJB : Ability category deleted from database =  " + abilityCategory.toString());
			
			result.put("error", "none");
		} else {
			result.put("error", "notEmpty");
			result.put("abilities", abilityCategory.getAbilities());
		}

		return result;
	}

	@Override
	@Transactional
	public void newAbilityCategory(Map<String, Object> newAbilityCategoryData)
			throws UnexpectedRollbackException {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		AbilityCategoryJPA abilityCategory = new AbilityCategoryJPA(newAbilityCategoryData.get("name").toString());
		
		List<HashMap> unformatedRequiredQuirkPairs = (List<HashMap>)newAbilityCategoryData.get("requiredQuirks");

		abilityCategory.setRequiredQuirks(new HashSet<AbilityCategoryQuirkJPA>());

		for(HashMap unformatedRequiredQuirkPair : unformatedRequiredQuirkPairs){

			QuirkJPA requiredQuirk = quirkEJB.getQuirkById(Integer.parseInt(unformatedRequiredQuirkPair.get("id").toString()));

			Boolean negated = Boolean.parseBoolean(unformatedRequiredQuirkPair.get("negated").toString());

			abilityCategory.getRequiredQuirks().add(new AbilityCategoryQuirkJPA(requiredQuirk, abilityCategory, negated));

		}

		entman.persist(abilityCategory);
		
		result.put("error", "none");
		
		logger.info("AbilityCategoryEJB : AbilityCategory addded to database =  " + abilityCategory.toString());
	}

	@Override
	public AbilityCategoryJPA getAbilityCategoryById(int id)
			throws NoResultException {
		
		AbilityCategoryJPA abilityCategory = entman.createQuery("from AbilityCategoryJPA a where a.id = ?", AbilityCategoryJPA.class).setParameter(1, id).getSingleResult();
		
		return abilityCategory;
	}

	@Override
	@Transactional
	public Map<String, Object> updateAbilityCategory(
			Map<String, Object> newAbilityCategoryData)
			throws PersistenceException {

		Map<String, Object> result = new HashMap<String, Object>();
		
		AbilityCategoryJPA oldAbilityCategoryData = entman.find(AbilityCategoryJPA.class, Integer.parseInt(newAbilityCategoryData.get("id").toString()));
		
		oldAbilityCategoryData.setName(newAbilityCategoryData.get("name").toString());
		
		//Aixo es un parxe perque, per algun motiu que desconec, oldAbilityCategoryData.getRequiredQuirks() no torna tots els requeriments
		List<AbilityCategoryQuirkJPA> oldRequirements = entman.createQuery("from AbilityCategoryQuirkJPA a where a.dependentAbilityCategoryJPA.id = ?",AbilityCategoryQuirkJPA.class).setParameter(1, Integer.parseInt(newAbilityCategoryData.get("id").toString())).getResultList();
		
		@SuppressWarnings("unchecked")
		List<HashMap<String,Object>> newRequiredQuirks = (List<HashMap<String,Object>>)newAbilityCategoryData.get("requiredQuirks");
		
		for (Iterator<AbilityCategoryQuirkJPA> iteratorOld = oldRequirements.iterator(); iteratorOld.hasNext();) {
			AbilityCategoryQuirkJPA oldRequiredQuirk = iteratorOld.next();
			
				boolean notFound = true;
				
				for (Iterator<HashMap<String,Object>> iteratorNew = newRequiredQuirks.iterator(); iteratorNew.hasNext();) {
				HashMap<String,Object> hashMapNew = iteratorNew.next();
				
					//Si se dona la igualtat, vol dir que coincideixen una de les condicions noves i una de les antigues, aleshores cal modificar unicament el valor de negated 
					if(Integer.parseInt(hashMapNew.get("id").toString()) == oldRequiredQuirk.getRequiredQuirkJPA().getId()){
						oldRequiredQuirk.setNegated(Boolean.parseBoolean(hashMapNew.get("negated").toString()));
						iteratorNew.remove();
						notFound = false;
						//Cal actualitzar els requriments de les habilitats pertanyents a aquesta categoria
						for(AbilityJPA ability: oldAbilityCategoryData.getAbilities()){
							for(AbilityQuirkJPA abilityQuirkJPA: ability.getRequiredQuirks()){
								if(oldRequiredQuirk.getRequiredQuirkJPA().equals(abilityQuirkJPA.getRequiredQuirkJPA())){
									abilityQuirkJPA.setNegated(oldRequiredQuirk.isNegated());
								}
							}
						}					
					}
				}	
				if(notFound){
					//Si una condicion antiga no esta entre les noves, vol dir que ha estat deseleccionada i per tant cal eliminar-la de la base de dades.
					iteratorOld.remove();
					entman.createQuery("delete from AbilityCategoryQuirkJPA q where q.id = ?)").setParameter(1, oldRequiredQuirk.getId()).executeUpdate();
					
					//S'esborra aquest quirk dels requeriments de totes les habilitats que pertanyen a aquesta categoria
					for(AbilityJPA ability: oldAbilityCategoryData.getAbilities()){
						for (Iterator<AbilityQuirkJPA> iterator = ability.getRequiredQuirks().iterator(); iterator.hasNext();) {
							AbilityQuirkJPA abilityQuirkJPA = iterator.next();
							if(oldRequiredQuirk.getRequiredQuirkJPA().equals(abilityQuirkJPA.getRequiredQuirkJPA())){
								iterator.remove();
								entman.createQuery("delete from AbilityQuirkJPA q where q.id = ?)").setParameter(1, abilityQuirkJPA.getId()).executeUpdate();
							}
						}
					}				
				}		
			}
			
		//Dins newRequiredQuirks nomes queden els quirks que no son antics, es a dir, son noves condicions i cal afegir-los a la base de dades
		
		for(HashMap<String,Object> newRequiredQuirk : newRequiredQuirks){
			QuirkJPA requiredQuirk = entman.find(QuirkJPA.class, Integer.parseInt(newRequiredQuirk.get("id").toString()));
			oldAbilityCategoryData.getRequiredQuirks().add(new AbilityCategoryQuirkJPA(requiredQuirk, oldAbilityCategoryData, Boolean.parseBoolean(newRequiredQuirk.get("negated").toString())));
			for(AbilityJPA ability: oldAbilityCategoryData.getAbilities()){
				ability.getRequiredQuirks().add(new AbilityQuirkJPA(requiredQuirk,ability,Boolean.parseBoolean(newRequiredQuirk.get("negated").toString())));
			}
		}
		
		entman.flush();
		
		//Ara cal actualitzar els personatges, si tenen habilitats que ara no son accessibles han de perdre'n la puntuacio
		
		List<CharacterAbilityJPA> charactersAbilityJPA = entman.createQuery("from CharacterAbilityJPA ac where ac.abilityJPA.abilityCategory.id = ?1",CharacterAbilityJPA.class).setParameter(1, Integer.parseInt(newAbilityCategoryData.get("id").toString())).getResultList();
		
		for(CharacterAbilityJPA characterAbilityJPA : charactersAbilityJPA){
		
			boolean nonAvailable = true;
		
			for (AbilityQuirkJPA abilityQuirk : characterAbilityJPA.getAbilityJPA().getRequiredQuirks()){
	
				if(abilityQuirk.isNegated() != characterAbilityJPA.getCharacterJPA().getQuirks().contains(abilityQuirk.getRequiredQuirkJPA())){
					nonAvailable = false;
					break;
	
				}
				
			}
			if (nonAvailable && !characterAbilityJPA.getAbilityJPA().getRequiredQuirks().isEmpty()){
				characterAbilityJPA.getCharacterJPA().setPendingRevission(true);
				entman.createQuery("delete from CharacterAbilityJPA ac where ac.id = ?)").setParameter(1, characterAbilityJPA.getId()).executeUpdate();
			}
	
		}

		result.put("error", "none");
		
		logger.info("AbilityCategoryEJB : AbilityCategory modified =  " + oldAbilityCategoryData.toString());
		
		logEJB.insertLog("Ability category modified: " + oldAbilityCategoryData.toString());
		
		return result;
		
	}
	
}
