package edu.uoc.characterbuilder.ejb;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;

import edu.uoc.characterbuilder.model.AbilityCategoryJPA;
import edu.uoc.characterbuilder.model.AbilityCategoryQuirkJPA;
import edu.uoc.characterbuilder.model.AbilityJPA;
import edu.uoc.characterbuilder.model.AbilityQuirkJPA;
import edu.uoc.characterbuilder.model.CharacterAbilityJPA;
import edu.uoc.characterbuilder.model.QuirkJPA;

@Named
@Stateless
public class AbilityEJB implements AbilityEJBLocal {

	private static final Logger logger = LoggerFactory
			.getLogger(AbilityEJB.class);

	@PersistenceContext(unitName = "characterBuilderPU")
	private EntityManager entman;
	
	@Inject
	private LogEJBLocal logEJB;
	
	@Inject
	private QuirkEJBLocal quirkEJB;
	
	@Inject
	private AbilityCategoryEJBLocal abilityCategoryEJB;
	
	@Override
	public List<AbilityJPA> getAllAbilities() {

		List<AbilityJPA> abilityList = entman.createQuery("from AbilityJPA a order by a.name",AbilityJPA.class).getResultList();

		return abilityList;
	}

	@Override
	@Transactional
	public Map<String, Object> deleteAbilityById(int id) throws org.springframework.transaction.UnexpectedRollbackException {
		
		AbilityJPA ability = entman.find(AbilityJPA.class, id);
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		entman.createQuery("delete from AbilityQuirkJPA q where q.dependentAbilityJPA.id = ?").setParameter(1, id).executeUpdate();
		entman.createQuery("delete from CharacterAbilityJPA ca where ca.abilityJPA.id = ?").setParameter(1, id).executeUpdate();
		entman.createQuery("delete from AbilityJPA q where q.id = ?").setParameter(1, id).executeUpdate();
		
		result.put("error", "none");
		
		logger.info("AbilityEJB : Ability deleted from database =  " + ability.toString());
		
		logEJB.insertLog("Ability deleted : " + ability.toString());

		return result;
	}

	@Override
	@Transactional
	public Map<String, Object> newAbility(Map<String, Object> newAbilityData)
			throws org.springframework.transaction.UnexpectedRollbackException {
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		AbilityCategoryJPA abilityCategory = abilityCategoryEJB.getAbilityCategoryById(Integer.parseInt(newAbilityData.get("abilityCategory").toString()));
	
		AbilityJPA ability = new AbilityJPA(newAbilityData.get("name").toString(), newAbilityData.get("description").toString(), Boolean.parseBoolean(newAbilityData.get("infancySelectable").toString()), abilityCategory);

		@SuppressWarnings("unchecked")
		List<HashMap<String,Object>> unformatedRequiredQuirkPairs = (List<HashMap<String,Object>>)newAbilityData.get("requiredQuirks");

		ability.setRequiredQuirks(new HashSet<AbilityQuirkJPA>());

		for(HashMap<String,Object> unformatedRequiredQuirkPair : unformatedRequiredQuirkPairs){

			QuirkJPA requiredQuirk = quirkEJB.getQuirkById(Integer.parseInt(unformatedRequiredQuirkPair.get("id").toString()));

			Boolean negated = Boolean.parseBoolean(unformatedRequiredQuirkPair.get("negated").toString());

			ability.getRequiredQuirks().add(new AbilityQuirkJPA(requiredQuirk, ability, negated));

		}
		//Els requeriments de les categories tambe ho son de les habilitats que en formen part
		//Aixo es un parxe perque, per algun motiu que desconec, oldAbilityCategoryData.getRequiredQuirks() no torna tots els requeriments
		List<AbilityCategoryQuirkJPA> requiredQuirks = entman.createQuery("from AbilityCategoryQuirkJPA a where a.dependentAbilityCategoryJPA.id = ?",AbilityCategoryQuirkJPA.class).setParameter(1, abilityCategory.getId()).getResultList();
		for(AbilityCategoryQuirkJPA abilityCategoryrequiredQuirk : requiredQuirks){
			ability.getRequiredQuirks().add(new AbilityQuirkJPA(abilityCategoryrequiredQuirk.getRequiredQuirkJPA(), ability, abilityCategoryrequiredQuirk.isNegated()));
		}
		
		entman.persist(ability);
		
		result.put("error", "none");
		
		logger.info("AbilityEJB : Ability addded to database =  " + abilityCategory.toString());
		
		return result;
		
	}

	@Override
	public AbilityJPA getAbilityById(int id)
			throws NoResultException {
		
		AbilityJPA ability = entman.createQuery("from AbilityJPA a where a.id = ?", AbilityJPA.class).setParameter(1, id).getSingleResult();
		
		return ability;
	}

	@Override
	@Transactional
	public Map<String, Object> updateAbility(
			Map<String, Object> newAbilityData)
			throws PersistenceException {

		Map<String, Object> result = new HashMap<String, Object>();
		
		AbilityJPA oldAbilityData = entman.find(AbilityJPA.class, Integer.parseInt(newAbilityData.get("id").toString()));
		
		AbilityCategoryJPA abilityCategory = abilityCategoryEJB.getAbilityCategoryById(Integer.parseInt(newAbilityData.get("abilityCategory").toString()));
		
		oldAbilityData.setName(newAbilityData.get("name").toString());
		oldAbilityData.setDescription(newAbilityData.get("description").toString());
		oldAbilityData.setInfancySelectable(Boolean.parseBoolean(newAbilityData.get("infancySelectable").toString()));
		
		
		//Primer se processen els requeriments directes(els checkboxes)
		
		@SuppressWarnings("unchecked")
		List<HashMap<String,Object>> newRequiredQuirks = (List<HashMap<String,Object>>)newAbilityData.get("requiredQuirks");
		
		for (Iterator<AbilityQuirkJPA> iteratorOld = oldAbilityData.getRequiredQuirks().iterator(); iteratorOld.hasNext();) {
			AbilityQuirkJPA oldRequiredQuirk = iteratorOld.next();
				
				boolean notFound = true;
				
				for (Iterator<HashMap<String,Object>> iteratorNew = newRequiredQuirks.iterator(); iteratorNew.hasNext();) {
				HashMap<String,Object> hashMapNew = iteratorNew.next();
				
					//Si se dona la igualtat, vol dir que coincideixen una de les condicions noves i una de les antigues, aleshores cal modificar unicament el valor de negated 
				
					if(Integer.parseInt(hashMapNew.get("id").toString()) == oldRequiredQuirk.getRequiredQuirkJPA().getId()){
						oldRequiredQuirk.setNegated(Boolean.parseBoolean(hashMapNew.get("negated").toString()));
						iteratorNew.remove();
						notFound = false;				
					}
				}	
				if(notFound){
					//Si una condicion antiga no esta entre les noves, vol dir que ha estat deseleccionada i per tant cal eliminar-la de la base de dades.
					iteratorOld.remove();
					entman.createQuery("delete from AbilityQuirkJPA q where q.id = ?)").setParameter(1, oldRequiredQuirk.getId()).executeUpdate();				
				}		
			}
			
		entman.flush();
		
		//Dins newRequiredQuirks nomes queden els quirks que no son antics, es a dir, son noves condicions i cal afegir-los a la base de dades
		
		for(HashMap<String,Object> newRequiredQuirk : newRequiredQuirks){
			QuirkJPA requiredQuirk = entman.find(QuirkJPA.class, Integer.parseInt(newRequiredQuirk.get("id").toString()));
			oldAbilityData.getRequiredQuirks().add(new AbilityQuirkJPA(requiredQuirk, oldAbilityData, Boolean.parseBoolean(newRequiredQuirk.get("negated").toString())));
		}
		
		/*
		Al modificar una habilitat pot passar que aquesta hagi canviat de categoria. Si es aixi, cal que perdi els requeriments de la categoria antiga
		i guanyi els de la nova.
		*/
		
		if(!abilityCategory.equals(oldAbilityData.getAbilityCategory())){
		
			//Aixo es un parxe perque, per algun motiu que desconec, oldAbilityCategoryData.getRequiredQuirks() no torna tots els requeriments
			List<AbilityCategoryQuirkJPA> oldRequirements = entman.createQuery("from AbilityCategoryQuirkJPA a where a.dependentAbilityCategoryJPA.id = ?",AbilityCategoryQuirkJPA.class).setParameter(1, oldAbilityData.getAbilityCategory().getId()).getResultList();
			
			for(AbilityCategoryQuirkJPA oldRequiredQuirk: oldRequirements){
				for (Iterator<AbilityQuirkJPA> iterator = oldAbilityData.getRequiredQuirks().iterator(); iterator.hasNext();) {
					AbilityQuirkJPA abilityQuirkJPA = iterator.next();
					if(abilityQuirkJPA.getRequiredQuirkJPA().equals(oldRequiredQuirk.getRequiredQuirkJPA())){
						entman.createQuery("delete from AbilityQuirkJPA q where q.id = ?)").setParameter(1, abilityQuirkJPA.getId()).executeUpdate();
						iterator.remove();
					}
				}
			}
			
			entman.flush();
			
			//Si afegim directament hi ha el risc de treure un requriment al bucle anterior i tornar-lo a afegir aqui. Aixo provoca un error de persistencia
			//La solucio es detectar les interseccions i tractar-les com modificacions
			//Se podria provar si falla de fer una interseccio de les dues llistes
					
			//Aixo es un parxe perque, per algun motiu que desconec, oldAbilityCategoryData.getRequiredQuirks() no torna tots els requeriments
			List<AbilityCategoryQuirkJPA> newRequirements = entman.createQuery("from AbilityCategoryQuirkJPA a where a.dependentAbilityCategoryJPA.id = ?",AbilityCategoryQuirkJPA.class).setParameter(1, abilityCategory.getId()).getResultList();
			
			for(AbilityCategoryQuirkJPA newRequiredQuirk: newRequirements){				
				oldAbilityData.getRequiredQuirks().add(new AbilityQuirkJPA(newRequiredQuirk.getRequiredQuirkJPA(),oldAbilityData, newRequiredQuirk.isNegated()));
			}
					
			oldAbilityData.setAbilityCategory(abilityCategory);
		}
		
		entman.flush();
		
		//Despres de modificar l'habilitat pot passar que ara sigui inaccessible per 1 o mes personatges. Per això, cal esborrar-la de les seves fitxes
		
		List<CharacterAbilityJPA> charactersAbilityJPA = entman.createQuery("from CharacterAbilityJPA ac where ac.abilityJPA.id = ?1",CharacterAbilityJPA.class).setParameter(1, oldAbilityData.getId()).getResultList();
		
		for(CharacterAbilityJPA characterAbilityJPA : charactersAbilityJPA){
		
			//Si l'habilitat no permet posar punts a la infantesa, cal esborrar-los de la base de dades
			System.out.println("Habilitat = " + characterAbilityJPA.getAbilityJPA().toString() + " - valor = " + oldAbilityData.isInfancySelectable());
			if(!oldAbilityData.isInfancySelectable()){
				System.out.println("Esborrant els infants = " + characterAbilityJPA.getId() + " - punts = " + characterAbilityJPA.getInfancyInvestedXps());
				characterAbilityJPA.setInfancyInvestedXps(0);
			}
			
			boolean nonAvailable = true;
		
			for (AbilityQuirkJPA abilityQuirk : characterAbilityJPA.getAbilityJPA().getRequiredQuirks()){
				
				if(abilityQuirk.isNegated() != characterAbilityJPA.getCharacterJPA().getQuirks().contains(abilityQuirk.getRequiredQuirkJPA())){
					nonAvailable = false;
					break;
	
				}
				
			}
			if (nonAvailable && !characterAbilityJPA.getAbilityJPA().getRequiredQuirks().isEmpty()){
				characterAbilityJPA.getCharacterJPA().setPendingRevission(true);
				entman.createQuery("delete from CharacterAbilityJPA ac where ac.id = ?)").setParameter(1, characterAbilityJPA.getId()).executeUpdate();
			}
	
		}
	
		result.put("error", "none");
		
		logger.info("AbilityEJB : Ability modified =  " + oldAbilityData.toString());
		
		logEJB.insertLog("Ability modified : " + oldAbilityData.toString());
		
		return result;
		
	}
	
}
