package edu.uoc.characterbuilder.ejb;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.transaction.UnexpectedRollbackException;
import org.springframework.transaction.annotation.Transactional;

import edu.uoc.characterbuilder.model.AbilityJPA;
import edu.uoc.characterbuilder.model.AbilityQuirkJPA;
import edu.uoc.characterbuilder.model.CharacterAbilityJPA;
import edu.uoc.characterbuilder.model.CharacterJPA;
import edu.uoc.characterbuilder.model.GenderEnum;
import edu.uoc.characterbuilder.model.QuirkDivisionEnum;
import edu.uoc.characterbuilder.model.QuirkJPA;
import edu.uoc.characterbuilder.model.QuirkQuirkJPA;
import edu.uoc.characterbuilder.model.UserCharacterJPA;
import edu.uoc.characterbuilder.model.UserJPA;

@Named
@Stateless
public class CharacterEJB implements CharacterEJBLocal {

	private static final Logger logger = LoggerFactory
			.getLogger(CharacterEJB.class);

	@PersistenceContext(unitName = "characterBuilderPU")
	private EntityManager entman;
	
	@Inject
	private UserManagementEJBLocal userEJB;
	
	@Inject
	private QuirkEJBLocal quirkEJB;
	
	@Inject
	private AbilityEJBLocal abilityEJB;

	@Override
	public List<CharacterJPA> getAccessibleCharactersByUserName(String userName) {
		
		List<CharacterJPA> characters = new ArrayList<CharacterJPA>();
		
		UserJPA user = userEJB.getUserByUsername(userName);
		
		characters = entman.createQuery("select c from CharacterJPA c join c.users u where u.userJPA.id = ? order by c.name",CharacterJPA.class).setParameter(1, user.getId()).getResultList();
		
		return characters;
	}

	@Override
	@Transactional
	public void deleteCharacterById(int id)
			throws UnexpectedRollbackException {
			
			//Degut a l'ordre per defecte en que executa les operacions no hi ha altre opcio que esborrar manualment aquets elements
			
			entman.createQuery("delete from UserCharacterJPA uc where uc.characterJPA.id = ? ").setParameter(1, id).executeUpdate();
			
			entman.createQuery("delete from CharacterAbilityJPA ca where ca.characterJPA.id = ? ").setParameter(1, id).executeUpdate();
			
			entman.flush();
			
			CharacterJPA characterJPA = entman.find(CharacterJPA.class, id);
			
			entman.remove(characterJPA);

			logger.info("CharacterEJB : Character deleted from database =  " + characterJPA.toString());
		
	}

	@Override
	@Transactional
	public void updateRights(HashMap<String, Object> accessRights) throws org.springframework.transaction.UnexpectedRollbackException {
		
		CharacterJPA characterJPA = entman.find(CharacterJPA.class, Integer.parseInt(accessRights.get("characterId").toString()));
		
		@SuppressWarnings("unchecked")
		List<HashMap<String,Object>> userRights =  (List<HashMap<String,Object>>)accessRights.get("userRights");
	
		for (Iterator<UserCharacterJPA> iteratorOld = characterJPA.getUsers().iterator(); iteratorOld.hasNext();) {
			UserCharacterJPA oldUserRight = iteratorOld.next();
			
			for (Iterator<HashMap<String,Object>> iteratorNew = userRights.iterator(); iteratorNew.hasNext();) {
				HashMap<String,Object> hashMap = iteratorNew.next();
				
				if(!oldUserRight.isOwner() && oldUserRight.getUserJPA().getId() == Integer.parseInt(hashMap.get("id").toString())){
					
					boolean canRead = Boolean.parseBoolean(hashMap.get("canRead").toString());
					boolean canWrite = Boolean.parseBoolean(hashMap.get("canWrite").toString());
					
					if(canRead == false && canWrite == false){
						iteratorOld.remove();
						entman.createQuery("delete from UserCharacterJPA uc where uc.userJPA.id = ? ").setParameter(1, oldUserRight.getUserJPA().getId()).executeUpdate();
					} else {
						oldUserRight.setCanRead(canRead);
						oldUserRight.setCanWrite(canWrite);
					}
					
					iteratorNew.remove();
					
				}
			}
		}
		
		for(HashMap<String,Object> newRight : userRights){
			
			UserJPA userJPA = entman.find(UserJPA.class, Integer.parseInt(newRight.get("id").toString()));
			
			boolean canRead = Boolean.parseBoolean(newRight.get("canRead").toString());
			boolean canWrite = Boolean.parseBoolean(newRight.get("canWrite").toString());
			
			characterJPA.getUsers().add(new UserCharacterJPA(userJPA,characterJPA,false,canRead,canWrite));
		}
		
		logger.info("CharacterEJB : updated character rights =  " + characterJPA.toString());
		
	}


	@Override
	public CharacterJPA getCharacterById(int id) throws NoResultException {
		
		CharacterJPA character = entman.find(CharacterJPA.class, id);
		
		return character;
	}

	@Override
	@Transactional
	public void makeUserOwner(Map<String, Object> data) {

		int characterId = Integer.parseInt(data.get("characterId").toString());
		int userId = Integer.parseInt(data.get("userId").toString());
		
		CharacterJPA characterJPA = entman.find(CharacterJPA.class, characterId);

		UserJPA newOwner = entman.find(UserJPA.class, userId);
		
		List<UserJPA> admins = userEJB.getAllAdmins();
		
		boolean notFound = true;
		
		for (Iterator<UserCharacterJPA> iterator = characterJPA.getUsers().iterator(); iterator.hasNext();) {
			UserCharacterJPA userCharacterJPA = iterator.next();
			
			//S'esborren els permisos de l'antic propietari
			if (userCharacterJPA.isOwner() && !admins.contains(userCharacterJPA.getUserJPA())) {
				iterator.remove();
				entman.createQuery("delete from UserCharacterJPA uc where uc.userJPA.id = ? ").setParameter(1, userCharacterJPA.getUserJPA().getId()).executeUpdate();
			}
			
			//Se modifiquen els permisos del nou propietari
			if (userCharacterJPA.getUserJPA().equals(newOwner)) {
				userCharacterJPA.setOwner(true);
				userCharacterJPA.setCanRead(true);
				userCharacterJPA.setCanWrite(true);
				notFound = false;
			}

		}
		
		if(notFound){
			characterJPA.getUsers().add(new UserCharacterJPA(newOwner, characterJPA, true, true, true));
		}
		
	}

	@Override
	@Transactional
	public void saveNewCharacter(Map<String, Object> newCharacterData) throws UnexpectedRollbackException {

		CharacterJPA newCharacter = buildCharacterJPA(newCharacterData);
		
		Authentication loggedUser = SecurityContextHolder.getContext().getAuthentication();
		
		//Asignacio dels propietaris: creador + admins
		
		List<UserJPA> owners = userEJB.getAllAdmins();
		
		UserJPA originalOwner = userEJB.getUserByUsername(loggedUser.getName());
		
		owners.add(originalOwner);
		
		newCharacter.setUsers(new HashSet<UserCharacterJPA>());
		
		for(UserJPA user : owners){
			newCharacter.getUsers().add(new UserCharacterJPA(user, newCharacter, true, true, true));
		}
		
		newCharacter.setPendingRevission(!validateCharacterJPA(newCharacter).isEmpty());
		
		entman.persist(newCharacter);
		
		logger.info("CharacterEJB : Character added to database =  " + newCharacter.toString());

	}

	@Override
	public List<String> validateCharacter(
			Map<String, Object> characterData) {
		
			CharacterJPA character = buildCharacterJPA(characterData);

		return validateCharacterJPA(character);
	}
	
	private List<String> validateQuirks(Set<QuirkJPA> chosenQuirks){

		List<String> result = new ArrayList<String>();
		
		int QuirksBalance = 0;
		
		for(QuirkJPA chosenQuirk: chosenQuirks){
			
			//Se mira si les virtuts i defectes estan equlibrats
			QuirksBalance += (chosenQuirk.getQuirkDivision().equals(QuirkDivisionEnum.VIRTUE))? chosenQuirk.getQuirkType().getCost() : -1 * chosenQuirk.getQuirkType().getCost();
			
			boolean requirementsNotMet = true;
			
			for (QuirkQuirkJPA quirkQuirk : chosenQuirk.getRequiredQuirks()){
				if(quirkQuirk.isNegated() != chosenQuirks.contains(quirkQuirk.getRequiredQuirkJPA())){
					requirementsNotMet = false;
					break;
				}
			}
			
			if (requirementsNotMet && !chosenQuirk.getRequiredQuirks().isEmpty()){
				result.add(chosenQuirk.getQuirkType() + " " + chosenQuirk.getQuirkDivision() + " " + chosenQuirk.getName() + ":Requeriments not met.");
			}
		}

		if(QuirksBalance != 0){
			result.add("Virtues and flaws are unbalanced.");
		}
		
		return result;
	}
	
	private boolean validateCharacteristics(CharacterJPA user){
		
		class innerClass {
	        public int charToCharPoints(int numChar){
	        	
	        	switch (numChar) {
		        case 1:  return -1;
		        case 2:  return -3;
		        case 3:  return -6;
		        case 0:  return 0;
		        case -1:  return 1;
		        case -2:  return 3;
		        case -3:  return 6;
		        default: return 0;
			}
	        }
	    }
		
		int initialChar = 7;
		
		innerClass helper = new innerClass();
		
		initialChar += helper.charToCharPoints(user.getIne());
		initialChar += helper.charToCharPoints(user.getPer());
		initialChar += helper.charToCharPoints(user.getCha());
		initialChar += helper.charToCharPoints(user.getCom());
		initialChar += helper.charToCharPoints(user.getDex());
		initialChar += helper.charToCharPoints(user.getQui());
		initialChar += helper.charToCharPoints(user.getStr());
		initialChar += helper.charToCharPoints(user.getSta());
		
		if(initialChar >= 0){
			return true;
		} else {
			return false;
		}
	}

	@Override
	@Transactional
	public void saveUpdatedCharacter(Map<String, Object> newCharacterData)
			throws UnexpectedRollbackException {
		
		CharacterJPA newCharacterVersion = buildCharacterJPA(newCharacterData);
		
		entman.createQuery("delete from CharacterAbilityJPA ca where ca.characterJPA.id = ? ").setParameter(1, Integer.parseInt(newCharacterData.get("id").toString())).executeUpdate();
		
		entman.flush();
		
		entman.merge(newCharacterVersion);
		
		logger.info("CharacterEJB : Character upddated =  " + newCharacterVersion.toString());

	}
	
	public List<String> validateCharacterJPA(CharacterJPA character){
		
		List<String> result = new ArrayList<String>();
		
		if(!validateCharacteristics(character)) {
			result.add("Error in characteristics: the spend amount of characteristic points is higher than 7");	
		}

		result.addAll(validateQuirks(character.getQuirks()));
		
		return result;
		
	}

	private CharacterJPA buildCharacterJPA(Map<String, Object> characterData) {
	
		CharacterJPA character = new CharacterJPA(characterData.get("name").toString(), 
				characterData.get("description").toString(),
				(characterData.get("gender").toString().equals("-1")? GenderEnum.FEMALE: GenderEnum.valueOf(characterData.get("gender").toString())),
				Integer.parseInt(characterData.get("age").toString()),
				Integer.parseInt(characterData.get("str").toString()),
				Integer.parseInt(characterData.get("sta").toString()),
				Integer.parseInt(characterData.get("per").toString()),
				Integer.parseInt(characterData.get("ine").toString()),
				Integer.parseInt(characterData.get("cha").toString()),
				Integer.parseInt(characterData.get("com").toString()),
				Integer.parseInt(characterData.get("dex").toString()),
				Integer.parseInt(characterData.get("qui").toString()));
		
		if(characterData.get("id") != null && !characterData.get("id").equals("")){
			character.setId(Integer.parseInt(characterData.get("id").toString()));
		}
		
		//Omplint la llista de quirks
		
		@SuppressWarnings("unchecked")
		List<String> chosenQuirks = new ArrayList<String>((List<String>)characterData.get("chosenQuirks"));
				
		if (chosenQuirks.isEmpty()){
			character.setQuirks(new HashSet<QuirkJPA>());
		} else {	
			
			List<Integer> ids = new ArrayList<Integer>();
			
			for (String chosenQuirk: chosenQuirks){
				ids.add(Integer.valueOf(chosenQuirk));
			}
			
			character.setQuirks(new HashSet<QuirkJPA>(entman.createQuery("from QuirkJPA q where q.id in ?1", QuirkJPA.class).setParameter(1, ids).getResultList()));
		}
		
		//Omplint la llista d'habilitats
		
		@SuppressWarnings("unchecked")
		List<Map<String,String>> characterAbilities = new ArrayList<Map<String,String>>((List<Map<String,String>>)characterData.get("abilities"));
		
		character.setAbilities(new HashSet<CharacterAbilityJPA>());
		
		for(Map<String,String> characterAbility : characterAbilities){
			
			AbilityJPA ability = entman.find(AbilityJPA.class, Integer.parseInt(characterAbility.get("id")));
			
			CharacterAbilityJPA characterAbilityJPA = new CharacterAbilityJPA(ability, character, Integer.parseInt(characterAbility.get("childHab")), Integer.parseInt(characterAbility.get("adultHab")));
			
			character.getAbilities().add(characterAbilityJPA);
		}
		
		//validacio
		
		character.setPendingRevission(!validateCharacterJPA(character).isEmpty());
		
		return character;
	}
	
	@Override
	public Map<String, Object> getUpdatedElementsLists(
			List<Integer> chosenQuirks) {	
		
		Map<String, Object> result = new HashMap<String, Object>();
		
		List<QuirkJPA> chosenQuirkJPA = null;
		
		List<QuirkJPA> nonAvailableQuirks = new ArrayList<QuirkJPA>();
		List<AbilityJPA> nonAvailableAbilities = new ArrayList<AbilityJPA>();
		
		if (chosenQuirks.isEmpty()){
			chosenQuirkJPA  = new ArrayList<QuirkJPA>();
		} else {	
			chosenQuirkJPA = entman.createQuery("from QuirkJPA q where q.id in ?1", QuirkJPA.class).setParameter(1, chosenQuirks).getResultList();
		}
		
		//Per tots els quirks existents se comprova si els quirks seleccionats els fan o no disponibles per triar
		
		List<QuirkJPA> allQuirks = quirkEJB.getAllQuirks();
		
		for(QuirkJPA quirk: allQuirks){
			
			boolean nonAvailable = true;
			
			for (QuirkQuirkJPA quirkQuirk : quirk.getRequiredQuirks()){
				if(quirkQuirk.isNegated() != chosenQuirkJPA.contains(quirkQuirk.getRequiredQuirkJPA())){
					nonAvailable = false;
					break;
				}
			}
			
			if (nonAvailable && !quirk.getRequiredQuirks().isEmpty()){
				nonAvailableQuirks.add(quirk);
			}
		}
		
		result.put("nonAvailableQuirks", nonAvailableQuirks);
		
		//Per tots les habilitats existents se comprova si els quirks seleccionats les fan o no disponibles per triar
		
		List<AbilityJPA> allAbilities = abilityEJB.getAllAbilities();
		
		for(AbilityJPA ability: allAbilities){
			
			boolean nonAvailable = true;
			
			for (AbilityQuirkJPA abilityQuirk : ability.getRequiredQuirks()){
				if(abilityQuirk.isNegated() != chosenQuirkJPA.contains(abilityQuirk.getRequiredQuirkJPA())){
					nonAvailable = false;
					break;
				}
			}
			
			if (nonAvailable && !ability.getRequiredQuirks().isEmpty()){
				nonAvailableAbilities.add(ability);
			}
		}
		
		result.put("nonAvailableAbilities", nonAvailableAbilities);
		
		return result;
	}
	
}
