package edu.uoc.characterbuilder.ejb;

import java.util.List;
import java.util.Map;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;

import edu.uoc.characterbuilder.model.QuirkCategoryJPA;
import edu.uoc.characterbuilder.model.QuirkJPA;

@Named
@Stateless
public class QuirkCategoryEJB implements QuirkCategoryEJBLocal {

	private static final Logger logger = LoggerFactory
			.getLogger(QuirkCategoryEJB.class);

	@PersistenceContext(unitName = "characterBuilderPU")
	private EntityManager entman;
	
	@Inject
	private LogEJBLocal log;

	@Override
	public QuirkCategoryJPA getQuirkCategoryById(int id) throws javax.persistence.NoResultException {
		
		QuirkCategoryJPA category = entman.createQuery("from QuirkCategoryJPA q where q.id = ?", QuirkCategoryJPA.class).setParameter(1, id).getSingleResult();
		
		return category;
	}
	
	@Transactional
	public void newQuirkCategory(Map<String, String> newQuirkCategoryData) throws org.springframework.transaction.UnexpectedRollbackException {

		QuirkCategoryJPA newQuirkCategory = new QuirkCategoryJPA(newQuirkCategoryData.get("name"));
		
		entman.persist(newQuirkCategory);
		
		logger.info("QuirkCategoryEJB : quirkCategory stored in database =  " + newQuirkCategory.toString());

	}

	@Override
	public List<QuirkCategoryJPA> getAllQuirkCategories() {

		List<QuirkCategoryJPA> quirkCategoriesList = entman.createQuery("from QuirkCategoryJPA q order by q.name",QuirkCategoryJPA.class).getResultList();

		return quirkCategoriesList;
	}

	@Override
	@Transactional
	public List<QuirkJPA> deleteQuirkCategoryById(int id) {
	
		QuirkCategoryJPA quirkCategory = entman.createQuery("from QuirkCategoryJPA q where q.id = ?", QuirkCategoryJPA.class).setParameter(1, id).getSingleResult();
		
		if (quirkCategory.getQuirks().isEmpty()) {
			entman.remove(quirkCategory);

			log.insertLog("Quirk category deleted : " + quirkCategory.toString());
			logger.info("QuirkCategoryEJB : Quirk category deleted from database =  " + quirkCategory.toString());
		} 

		return quirkCategory.getQuirks();
		
	}

	@Override
	@Transactional
	public void updateQuirkCategory(Map<String, String> newQuirkCategoryData) throws javax.persistence.PersistenceException {

		QuirkCategoryJPA oldQuirkCategoryData = entman.find(QuirkCategoryJPA.class, Integer.parseInt(newQuirkCategoryData.get("id")));
		
		String oldname = oldQuirkCategoryData.getName();
		
		oldQuirkCategoryData.setName(newQuirkCategoryData.get("name"));
		
		entman.flush();
		
		log.insertLog("Quirk category updated : " + oldQuirkCategoryData.toString());
		logger.info("QuirkCategoryEJB : Quirk category " + oldname + " has been updated.");
		
	}
	
}
