package edu.uoc.characterbuilder.ejb;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.UnexpectedRollbackException;
import org.springframework.transaction.annotation.Transactional;

import edu.uoc.characterbuilder.model.AbilityQuirkJPA;
import edu.uoc.characterbuilder.model.CharacterAbilityJPA;
import edu.uoc.characterbuilder.model.CharacterJPA;
import edu.uoc.characterbuilder.model.QuirkCategoryJPA;
import edu.uoc.characterbuilder.model.QuirkDivisionEnum;
import edu.uoc.characterbuilder.model.QuirkJPA;
import edu.uoc.characterbuilder.model.QuirkQuirkJPA;
import edu.uoc.characterbuilder.model.QuirkTypeEnum;

@Named
@Stateless
public class QuirkEJB implements QuirkEJBLocal {

	private static final Logger logger = LoggerFactory
			.getLogger(QuirkEJB.class);

	@PersistenceContext(unitName = "characterBuilderPU")
	private EntityManager entman;
	
	@Inject
	private LogEJBLocal log;
	
	@Inject
	private QuirkCategoryEJBLocal quirkCategory;
	
	@Inject
	private CharacterEJBLocal characterEJB;

	@Override
	@Transactional
	public void deleteQuirkById(int id) {
		
		QuirkJPA quirk = entman.find(QuirkJPA.class, id);

		entman.createQuery("delete from QuirkQuirkJPA q where ( q.requiredQuirkJPA.id = ?1 or q.dependentQuirkJPA.id = ?2)").setParameter(1, id).setParameter(2, id).executeUpdate();
		entman.createQuery("delete from AbilityCategoryQuirkJPA a where a.requiredQuirkJPA.id = ?").setParameter(1, id).executeUpdate();
		entman.createQuery("delete from AbilityQuirkJPA a where a.requiredQuirkJPA.id = ?").setParameter(1, id).executeUpdate();

		//Com que characterJPA es la propietaria de la relacio quirk-character, si s'ha d'esborrar desde quirk cal fer-ho manualment
		
		for(CharacterJPA character: quirk.getCharacters()){
			character.getQuirks().remove(quirk);
			character.setPendingRevission(!characterEJB.validateCharacterJPA(character).isEmpty());
		}
		
		entman.flush();
		
		entman.createQuery("delete from QuirkJPA q where q.id = ?").setParameter(1, id).executeUpdate();

		log.insertLog("Quirk deleted: " + quirk.toString());
		logger.info("QuirkEJB : Quirk deleted from database =  " + quirk.toString());

	}

	@Override
	@Transactional
	public void newQuirk(Map<String, Object> newQuirkData) throws UnexpectedRollbackException {
	
		
	QuirkJPA quirk = new QuirkJPA(newQuirkData.get("name").toString(),
								newQuirkData.get("description").toString(), 
								QuirkDivisionEnum.valueOf(newQuirkData.get("quirkDivision").toString()),
								QuirkTypeEnum.valueOf(newQuirkData.get("quirkType").toString()),
								quirkCategory.getQuirkCategoryById(Integer.parseInt(newQuirkData.get("quirkCategory").toString())));
	
	
		List<HashMap> unformatedQuirkPairs = (List<HashMap>)newQuirkData.get("requiredQuirks");
		
		quirk.setRequiredQuirks(new HashSet<QuirkQuirkJPA>());
	
		for(HashMap unformatedQuirkPair : unformatedQuirkPairs){

			QuirkJPA requiredQuirk = getQuirkById(Integer.parseInt(unformatedQuirkPair.get("id").toString()));
			
			Boolean negated = Boolean.parseBoolean(unformatedQuirkPair.get("negated").toString());
			
			quirk.getRequiredQuirks().add(new QuirkQuirkJPA(negated, requiredQuirk, quirk));

		}
		
		entman.persist(quirk);
		
		logger.info("QuirkEJB : Quirk added to database =  " + quirk.toString());
		
	}

	@Override
	public QuirkJPA getQuirkById(int id) throws NoResultException {
		
		QuirkJPA quirk = entman.createQuery("from QuirkJPA q where q.id = ?", QuirkJPA.class).setParameter(1, id).getSingleResult();
		
		return quirk;
	}

	@Override
	public List<QuirkJPA> getAllQuirks() {
		
		List<QuirkJPA> quirkList = entman.createQuery("from QuirkJPA q order by q.name",QuirkJPA.class).getResultList();

		return quirkList;
	}

	@Override
	@Transactional
	public void updateQuirk(Map<String, Object> newQuirkData)
			throws PersistenceException {

		QuirkJPA oldQuirkData = entman.find(QuirkJPA.class, Integer.parseInt(newQuirkData.get("id").toString()));
		
		oldQuirkData.setName(newQuirkData.get("name").toString());
		oldQuirkData.setDescription(newQuirkData.get("description").toString());
		oldQuirkData.setQuirkDivision(QuirkDivisionEnum.valueOf(newQuirkData.get("quirkDivision").toString()));
		oldQuirkData.setQuirkType(QuirkTypeEnum.valueOf(newQuirkData.get("quirkType").toString()));
		
		QuirkCategoryJPA category = entman.find(QuirkCategoryJPA.class, Integer.parseInt(newQuirkData.get("quirkCategory").toString()));
		
		oldQuirkData.setQuirkCategory(category);
		
		List<HashMap> unformatedQuirkPairs = (List<HashMap>)newQuirkData.get("requiredQuirks");
		
		if (oldQuirkData.getRequiredQuirks() == null) {
			oldQuirkData.setRequiredQuirks(new HashSet<QuirkQuirkJPA>());
		} else {
			entman.createQuery("delete from QuirkQuirkJPA q where ( q.requiredQuirkJPA.id = ?1 or q.dependentQuirkJPA.id = ?2)").setParameter(1, oldQuirkData.getId()).setParameter(2, oldQuirkData.getId()).executeUpdate();
			oldQuirkData.getRequiredQuirks().clear();
		}
	
		for(HashMap unformatedQuirkPair : unformatedQuirkPairs){

			QuirkJPA requiredQuirk = getQuirkById(Integer.parseInt(unformatedQuirkPair.get("id").toString()));
			
			Boolean negated = Boolean.parseBoolean(unformatedQuirkPair.get("negated").toString());
			
			oldQuirkData.getRequiredQuirks().add(new QuirkQuirkJPA(negated, requiredQuirk, oldQuirkData));

		}
		
		entman.flush();
		
		List<CharacterJPA> characters = entman.createQuery("from CharacterJPA c where (from QuirkJPA q where q.id = ?1) member of c.quirks",CharacterJPA.class).setParameter(1, oldQuirkData.getId()).getResultList();
		
		for(CharacterJPA character: characters){
			
			character.setPendingRevission(!characterEJB.validateCharacterJPA(character).isEmpty());
			
		}
	
		log.insertLog("Quirk updated: " + oldQuirkData.toString());
		logger.info("QuirkEJB : Quirk deleted from database =  " + oldQuirkData.toString());
		
	}
	
}
