package edu.uoc.characterbuilder.ejb;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.ejb.Stateless;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;

import edu.uoc.characterbuilder.model.RoleJPA;
import edu.uoc.characterbuilder.model.UserJPA;

/**
 * Session Bean implementation class AdministracioUsuarisEJB
 */
@Named
@Stateless
public class UserManagementEJB implements UserManagementEJBLocal {

	private static final Logger logger = LoggerFactory
			.getLogger(UserManagementEJB.class);

	@PersistenceContext(unitName = "characterBuilderPU")
	private EntityManager entman;

	@Transactional
	public void newUser(Map<String, String> newUserData) throws org.springframework.transaction.UnexpectedRollbackException {

		UserJPA newUser = new UserJPA(newUserData.get("username"), newUserData.get("password"), newUserData.get("email"));
		
		List<RoleJPA> listRoles = new ArrayList<RoleJPA>();
		
		RoleJPA role = entman.createQuery("from RoleJPA u where u.name = ?", RoleJPA.class).setParameter(1, "ROLE_PLAYER").getSingleResult();
		
		listRoles.add(role);
		
		newUser.setRoles(listRoles);
		
		entman.persist(newUser);
		
		logger.info("UserManagementEJB : User stored in database =  " + newUser.toString());

	}

	@Override
	public UserJPA getUserByUsername(String username) throws javax.persistence.NoResultException {
		
		UserJPA user = entman.createQuery("from UserJPA u where u.username = ?", UserJPA.class).setParameter(1, username).getSingleResult();
		
		return user;
	}

	@Override
	public List<UserJPA> getAllPLayers() {
		
		/*La query se podria haver escrit com "from UserJPA u where 1 not member of u.roles" pero no m'agrada la idea de fer el codi depenent del valor
		que una sequencia doni a un registre. L'alternativa es declarar UserJPA.name com clau primaria pero aixo en complicaria la modificació*/
		
		@SuppressWarnings("unchecked")
		List<UserJPA> playerList = entman.createQuery("from UserJPA u where (from RoleJPA r where r.name = 'ROLE_ADMIN') not member of u.roles order by u.username").getResultList();

		return playerList;
	}

	@Override
	public List<UserJPA> getAllUsers() {
		
		@SuppressWarnings("unchecked")
		List<UserJPA> userList = entman.createQuery("from UserJPA u where order by u.username").getResultList();

		return userList;
	}
	
	@Override
	public List<UserJPA> getAllAdmins() {
		
		@SuppressWarnings("unchecked")
		List<UserJPA> adminList = entman.createQuery("from UserJPA u where (from RoleJPA r where r.name = 'ROLE_ADMIN') member of u.roles order by u.username").getResultList();

		return adminList;
	}
	
	@Override
	@Transactional
	public void deleteUserById(int id) {
		
		UserJPA user = entman.createQuery("from UserJPA where id = ?", UserJPA.class).setParameter(1, id).getSingleResult();
		
		entman.createQuery("delete from UserCharacterJPA uc where uc.userJPA.id = ? ").setParameter(1, id).executeUpdate();
		
		entman.remove(user);
		
		logger.info("UserManagementEJB : User deleted from database =  " + user.toString());
		
	}

	@Override
	@Transactional
	public void updateUser(Map<String, String> newUserData) throws javax.persistence.PersistenceException {

		UserJPA oldUserData = entman.find(UserJPA.class, Integer.parseInt(newUserData.get("id")));
		
		oldUserData.setUsername(newUserData.get("username"));
		if (newUserData.get("password") != null && !newUserData.get("password").equals("")){
			oldUserData.setPassword(newUserData.get("password"));
		}
		oldUserData.setEmail(newUserData.get("email"));
		
		entman.flush();
		
		logger.info("UserManagementEJB : User updated =  " + oldUserData.toString());
		
	}

	@Override
	public List<UserJPA> getNonOwners(int id) {
		
		List<UserJPA> owners = entman.createQuery("select uc.userJPA from UserCharacterJPA uc where uc.characterJPA.id = ?1 and uc.owner = 'true'", UserJPA.class).setParameter(1, id).getResultList();
		
		List<UserJPA> allPlayers = getAllPLayers();
		
		allPlayers.removeAll(owners);
		
		return allPlayers;
	}
	
}
