package edu.uoc.characterbuilder.model;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

import com.fasterxml.jackson.annotation.JsonIgnore;

@Entity
@Table(name="ability",schema="characterbuilder")
public class AbilityJPA implements Serializable {

	private static final long serialVersionUID = 1L;
	
	@SequenceGenerator(name="ability_id_seq",sequenceName="characterbuilder.ability_id_seq", allocationSize=1)
	@GeneratedValue(generator="ability_id_seq",strategy=GenerationType.SEQUENCE)
	@Id
	private int id;
	private String name;
	private String description;
	private boolean infancySelectable;

	@ManyToOne(fetch = FetchType.EAGER )
	@JoinTable(name="ability_abilitycategory",schema="characterbuilder",
		      joinColumns={@JoinColumn(name="id_ability", referencedColumnName="id")},
		      inverseJoinColumns={@JoinColumn(name="id_abilitycategory", referencedColumnName="id")})
	private AbilityCategoryJPA abilityCategory;
	@OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL,mappedBy="dependentAbilityJPA")
	@JsonIgnore
	private Set<AbilityQuirkJPA> requiredQuirks;

	protected AbilityJPA() {
		super();
	}

	public AbilityJPA(String name, String description,
			boolean infancySelectable, AbilityCategoryJPA abilityCategory) {
		super();
		this.name = name;
		this.description = description;
		this.infancySelectable = infancySelectable;
		this.abilityCategory = abilityCategory;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public boolean isInfancySelectable() {
		return infancySelectable;
	}

	public void setInfancySelectable(boolean infancySelectable) {
		this.infancySelectable = infancySelectable;
	}

	public AbilityCategoryJPA getAbilityCategory() {
		return abilityCategory;
	}

	public void setAbilityCategory(AbilityCategoryJPA abilityCategory) {
		this.abilityCategory = abilityCategory;
	}

	public Set<AbilityQuirkJPA> getRequiredQuirks() {
		return requiredQuirks;
	}

	public void setRequiredQuirks(Set<AbilityQuirkJPA> requiredQuirks) {
		this.requiredQuirks = requiredQuirks;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((abilityCategory == null) ? 0 : abilityCategory.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AbilityJPA other = (AbilityJPA) obj;
		if (abilityCategory == null) {
			if (other.abilityCategory != null)
				return false;
		} else if (!abilityCategory.equals(other.abilityCategory))
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return abilityCategory.getName() + " ability " + name;
	}
	
}
