package edu.uoc.characterbuilder.model;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

import com.fasterxml.jackson.annotation.JsonIgnore;

@Entity
@Table(name="character",schema="characterbuilder")
public class CharacterJPA implements Serializable {
	
	private static final long serialVersionUID = 1L;
	@SequenceGenerator(name="character_id_seq",sequenceName="characterbuilder.character_id_seq", allocationSize=1)
	@GeneratedValue(generator="character_id_seq",strategy=GenerationType.SEQUENCE)
	@Id
	private int id;
	@Column(name = "name")
	private String name;
	@Column(name = "description")
	private String description;
	@Column(name = "gender")
	@Enumerated(EnumType.STRING)
	private GenderEnum gender;
	@Column(name = "age")
	private int age;
	@Column(name = "str")
	private int str;
	@Column(name = "sta")
	private int sta;
	@Column(name = "per")
	private int per;
	@Column(name = "ine")
	private int ine;
	@Column(name = "cha")
	private int cha;
	@Column(name = "com")
	private int com;
	@Column(name = "dex")
	private int dex;
	@Column(name = "qui")
	private int qui;
	@Column(name = "pendingrevission")
	private boolean pendingRevission = false;
	
	@OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL,mappedBy="characterJPA")
	private Set<CharacterAbilityJPA> abilities;
	@JsonIgnore
	@OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL,mappedBy="characterJPA")
	private Set<UserCharacterJPA> users;
	@ManyToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER )
	@JoinTable(name="character_quirk",schema="characterbuilder",
		      joinColumns={@JoinColumn(name="id_character", referencedColumnName="id")},
		      inverseJoinColumns={@JoinColumn(name="id_quirk", referencedColumnName="id")})
	private Set<QuirkJPA> quirks;
	
	protected CharacterJPA() {

	}

	public CharacterJPA(String name, String description, GenderEnum gender,
			int age, int str, int sta, int per, int ine, int cha, int com,
			int dex, int qui) {
		super();
		this.name = name;
		this.description = description;
		this.gender = gender;
		this.age = age;
		this.str = str;
		this.sta = sta;
		this.per = per;
		this.ine = ine;
		this.cha = cha;
		this.com = com;
		this.dex = dex;
		this.qui = qui;
	}



	public int getId() {
		return id;
	}
	public void setId(int id) {
		this.id = id;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public String getDescription() {
		return description;
	}
	public void setDescription(String description) {
		this.description = description;
	}
	public GenderEnum getGender() {
		return gender;
	}
	public void setGender(GenderEnum gender) {
		this.gender = gender;
	}
	public int getAge() {
		return age;
	}
	public void setAge(int age) {
		this.age = age;
	}
	public int getStr() {
		return str;
	}
	public void setStr(int str) {
		this.str = str;
	}
	public int getSta() {
		return sta;
	}
	public void setSta(int sta) {
		this.sta = sta;
	}
	public int getPer() {
		return per;
	}
	public void setPer(int per) {
		this.per = per;
	}
	public int getIne() {
		return ine;
	}
	public void setIne(int ine) {
		this.ine = ine;
	}
	public int getCha() {
		return cha;
	}
	public void setCha(int cha) {
		this.cha = cha;
	}
	public int getCom() {
		return com;
	}
	public void setCom(int com) {
		this.com = com;
	}
	public int getDex() {
		return dex;
	}
	public void setDex(int dex) {
		this.dex = dex;
	}
	public int getQui() {
		return qui;
	}
	public void setQui(int qui) {
		this.qui = qui;
	}
	public boolean isPendingRevission() {
		return pendingRevission;
	}
	public void setPendingRevission(boolean pendingRevission) {
		this.pendingRevission = pendingRevission;
	}
	public Set<CharacterAbilityJPA> getAbilities() {
		return abilities;
	}
	public void setAbilities(Set<CharacterAbilityJPA> abilities) {
		this.abilities = abilities;
	}
	public Set<UserCharacterJPA> getUsers() {
		return users;
	}
	public void setUsers(Set<UserCharacterJPA> users) {
		this.users = users;
	}
	public Set<QuirkJPA> getQuirks() {
		return quirks;
	}
	public void setQuirks(Set<QuirkJPA> quirks) {
		this.quirks = quirks;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CharacterJPA other = (CharacterJPA) obj;
		if (id != other.id)
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "Character name = " + name + ", description = " + description
				+ ", gender = " + gender;
	}
	
	
	
}
