package edu.uoc.characterbuilder.model;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

import com.fasterxml.jackson.annotation.JsonIgnore;

@Entity
@Table(name="quirk",schema="characterbuilder")
public class QuirkJPA implements Serializable{

	private static final long serialVersionUID = 1L;
	@SequenceGenerator(name="quirk_id_seq",sequenceName="characterbuilder.quirk_id_seq", allocationSize=1)
	@GeneratedValue(generator="quirk_id_seq",strategy=GenerationType.SEQUENCE)
	@Id
	private int id;
	private String name;
	private String description;
	@Enumerated(EnumType.STRING)
	private QuirkDivisionEnum quirkDivision;
	@Enumerated(EnumType.STRING)
	private QuirkTypeEnum quirkType;
	@ManyToOne(fetch = FetchType.EAGER )
	@JoinTable(name="quirkcategory_quirk",schema="characterbuilder",
		      joinColumns={@JoinColumn(name="id_quirk", referencedColumnName="id")},
		      inverseJoinColumns={@JoinColumn(name="id_quirkcategory", referencedColumnName="id")})
	private QuirkCategoryJPA quirkCategory;

	@JsonIgnore
	@ManyToMany(mappedBy = "quirks")
	private Set<CharacterJPA> characters;
	
	@OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL,mappedBy="dependentQuirkJPA")
	@JsonIgnore
	private Set<QuirkQuirkJPA> requiredQuirks;

	@OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL,mappedBy="requiredQuirkJPA")
	@JsonIgnore
	private Set<QuirkQuirkJPA> dependentQuirks;
	
	@OneToMany(fetch = FetchType.EAGER, cascade = {CascadeType.DETACH,CascadeType.MERGE, CascadeType.REFRESH, CascadeType.REMOVE},mappedBy="requiredQuirkJPA")
	@JsonIgnore
	private Set<AbilityCategoryQuirkJPA> dependentAbilityCategory;

	@OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL,mappedBy="requiredQuirkJPA")
	@JsonIgnore
	private Set<AbilityQuirkJPA> dependentAbility;

	protected QuirkJPA() {

	}
	
	public QuirkJPA(String name, String description,
			QuirkDivisionEnum quirkDivision, QuirkTypeEnum quirkType,
			QuirkCategoryJPA quirkCategory) {
		super();
		this.name = name;
		this.description = description;
		this.quirkDivision = quirkDivision;
		this.quirkType = quirkType;
		this.quirkCategory = quirkCategory;
	}

	public int getId() {
		return id;
	}
	public void setId(int id) {
		this.id = id;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public String getDescription() {
		return description;
	}
	public void setDescription(String description) {
		this.description = description;
	}
	public QuirkTypeEnum getQuirkType() {
		return quirkType;
	}
	public void setQuirkType(QuirkTypeEnum quirkType) {
		this.quirkType = quirkType;
	}
	public QuirkCategoryJPA getQuirkCategory() {
		return quirkCategory;
	}
	public void setQuirkCategory(QuirkCategoryJPA quirkCategory) {
		this.quirkCategory = quirkCategory;
	}
	public QuirkDivisionEnum getQuirkDivision() {
		return quirkDivision;
	}
	public void setQuirkDivision(QuirkDivisionEnum quirkDivision) {
		this.quirkDivision = quirkDivision;
	}
	public Set<AbilityCategoryQuirkJPA> getDependentAbilityCategory() {
		return dependentAbilityCategory;
	}
	public void setDependentAbilityCategory(
			Set<AbilityCategoryQuirkJPA> dependentAbilityCategory) {
		this.dependentAbilityCategory = dependentAbilityCategory;
	}
	public Set<AbilityQuirkJPA> getDependentAbility() {
		return dependentAbility;
	}
	public void setDependentAbility(Set<AbilityQuirkJPA> dependentAbility) {
		this.dependentAbility = dependentAbility;
	}
	public Set<QuirkQuirkJPA> getDependentQuirks() {
		return dependentQuirks;
	}
	public void setDependentQuirks(Set<QuirkQuirkJPA> dependentQuirks) {
		this.dependentQuirks = dependentQuirks;
	}
	public Set<QuirkQuirkJPA> getRequiredQuirks() {
		return requiredQuirks;
	}
	public void setRequiredQuirks(Set<QuirkQuirkJPA> requiredQuirks) {
		this.requiredQuirks = requiredQuirks;
	}
	public Set<CharacterJPA> getCharacters() {
		return characters;
	}
	public void setCharacters(Set<CharacterJPA> characters) {
		this.characters = characters;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result
				+ ((quirkCategory == null) ? 0 : quirkCategory.hashCode());
		result = prime * result
				+ ((quirkDivision == null) ? 0 : quirkDivision.hashCode());
		result = prime * result
				+ ((quirkType == null) ? 0 : quirkType.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		QuirkJPA other = (QuirkJPA) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (quirkCategory == null) {
			if (other.quirkCategory != null)
				return false;
		} else if (!quirkCategory.equals(other.quirkCategory))
			return false;
		if (quirkDivision != other.quirkDivision)
			return false;
		if (quirkType != other.quirkType)
			return false;
		return true;
	}

	@Override
	public String toString() {
		return quirkType + " " + quirkCategory.getName() + " " + quirkDivision + " " + name;
	}
	
}
