package edu.uoc.characterbuilder.model;

import java.io.Serializable;
import java.util.List;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

import com.fasterxml.jackson.annotation.JsonIgnore;

@Entity
@Table(name="user",schema="characterbuilder")
public class UserJPA implements Serializable {
	
	//La sequencia user_id_seq se genera automaticament al definir un camp com serial. Rep aquest nom automaticament.
	//El allocationSize=1 és per evitar que, amb postgresql, Hibernate parteixi de 50.
	private static final long serialVersionUID = 1L;
	@SequenceGenerator(name="user_id_seq",sequenceName="characterbuilder.user_id_seq", allocationSize=1)
	@GeneratedValue(generator="user_id_seq",strategy=GenerationType.SEQUENCE)
	@Id
	private int id;
	private String username;
	@JsonIgnore
	private String password;
	private String email;
	@ManyToMany(cascade = {CascadeType.MERGE, CascadeType.REFRESH, CascadeType.PERSIST}, fetch = FetchType.EAGER )
	@JoinTable(name="user_role",schema="characterbuilder",
		      joinColumns={@JoinColumn(name="id_user", referencedColumnName="id")},
		      inverseJoinColumns={@JoinColumn(name="id_role", referencedColumnName="id")})
	private List<RoleJPA> roles;
	
	@OneToMany(fetch = FetchType.EAGER, cascade = {CascadeType.MERGE, CascadeType.REFRESH, CascadeType.PERSIST},mappedBy="userJPA")
	private Set<UserCharacterJPA> characters;
	
	protected UserJPA(){
	}
	
	public UserJPA(String username, String password, String email) {
	super();
	this.username = username;
	this.password = password;
	this.email = email;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public List<RoleJPA> getRoles() {
		return roles;
	}

	public void setRoles(List<RoleJPA> roles) {
		this.roles = roles;
	}

	public Set<UserCharacterJPA> getCharacters() {
		return characters;
	}

	public void setCharacters(Set<UserCharacterJPA> characters) {
		this.characters = characters;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		result = prime * result
				+ ((username == null) ? 0 : username.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		UserJPA other = (UserJPA) obj;
		if (id != other.id)
			return false;
		if (username == null) {
			if (other.username != null)
				return false;
		} else if (!username.equals(other.username))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "User username = " + username + ", email = " + email;
	}
	
}
