package edu.uoc.characterbuilder.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.inject.Inject;
import javax.persistence.NoResultException;

import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import edu.uoc.characterbuilder.ejb.UserManagementEJBLocal;
import edu.uoc.characterbuilder.model.RoleJPA;
import edu.uoc.characterbuilder.model.UserJPA;

@Service
public class CustomUserDetailsService implements UserDetailsService {

	@Inject
	private UserManagementEJBLocal userEJB;
	
	@Override
	public UserDetails loadUserByUsername(String login) throws UsernameNotFoundException {

        UserJPA user = null;
		try {
			user = userEJB.getUserByUsername(login);
		} catch (NoResultException e) {
			throw new UsernameNotFoundException(null);
		}

        boolean enabled = true;
        boolean accountNonExpired = true;
        boolean credentialsNonExpired = true;
        boolean accountNonLocked = true;

        return new User(
        		user.getUsername(),
                user.getPassword(),
                enabled,
                accountNonExpired,
                credentialsNonExpired,
                accountNonLocked,
                getAuthorities(user.getRoles())
        );
    }

    public Collection<? extends GrantedAuthority> getAuthorities(List<RoleJPA> roles) {
    	  	
    	List<GrantedAuthority> authorities = new ArrayList<GrantedAuthority>();
    	
    	for(RoleJPA role: roles){
    		authorities.add(new SimpleGrantedAuthority(role.getName()));
    	}

        return authorities;
    }

}
